//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers    1
#property indicator_color1     Red

extern string TimeFrame      = "Current time frame";
extern double Lambda         = 1600;
extern int    MomentumPeriod = 1;
extern bool   verticalLinesVisible   = false;
extern string verticalLinesID        = "HPVelo1";
extern color  verticalLinesUpColor   = LimeGreen;
extern color  verticalLinesDnColor   = Orange;
extern int    verticalLinesStyle     = STYLE_DOT;
extern int    verticalLinesWidth     = 0;

//
//
//
//
//

double hp[];
double cross[];
double momentum[];
int    timeFrame;
string name;
int    window;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(3);
   SetIndexBuffer(0,momentum);
   SetIndexBuffer(1,hp);
   SetIndexBuffer(2,cross);
      timeFrame = stringToTimeFrame(TimeFrame);
      name = verticalLinesID+" "+timeFrameToString(timeFrame)+" HP Filter Velo NORPT V2";
      IndicatorShortName(name);
   return(0);
}
int deinit()
{
   string lookFor       = verticalLinesID+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//
//
//
//
//

int start()
{
    int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
         int limit = MathMin(Bars-counted_bars,Bars-1);
         window = WindowFind(name);

   for(int i=MathMax(limit-1,0); i>=0; i--)  
   {  
      int y = iBarShift(NULL,timeFrame,Time[i]);
         hp[i] = iCustom(NULL,timeFrame,"HP",Lambda,0,y);
         momentum[i] = hp[i]-iCustom(NULL,timeFrame,"HP",Lambda,0,y+MomentumPeriod);
         cross[i] = cross[i+1];
            if (momentum[i]>0) cross[i] =  1;
            if (momentum[i]<0) cross[i] = -1;
            if (verticalLinesVisible)
            {
               ObjectDelete(verticalLinesID+":u:"+Time[i]);
               ObjectDelete(verticalLinesID+":d:"+Time[i]);
               if (cross[i] != cross[i+1])
               {
                  if (cross[i]== -1) drawLine(":d:",i,verticalLinesDnColor);
                  if (cross[i]==  1) drawLine(":u:",i,verticalLinesUpColor);
               }
            }            
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void drawLine(string add, int i,color theColor)
{
   if (window==-1) return;
   string name = verticalLinesID+add+Time[i];
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_VLINE,window,Time[i],0);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_STYLE,verticalLinesStyle);
         ObjectSet(name,OBJPROP_WIDTH,verticalLinesWidth);
         ObjectSet(name,OBJPROP_BACK,true);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}